# Python modules
import time
import hashlib

# 3rd party modules
import posix_ipc

# Utils for this demo
import utils


utils.say("Oooo 'ello, I'm Mrs. Premise!")

params = utils.read_params()

# Create the message queue.
mq = posix_ipc.MessageQueue(params["MESSAGE_QUEUE_NAME"], posix_ipc.O_CREX)

# The first message is a random string (the current time).
s = time.asctime()
utils.say("Sending %s" % s)
mq.send(s)
what_i_sent = s

for i in range(0, params["ITERATIONS"]):
    utils.say("iteration %d" % i)

    s, _ = mq.receive()
    s = s.decode()
    utils.say("Received %s" % s)

    # If the message is what I wrote, put it back on the queue.
    while s == what_i_sent:
        # Nothing new; give Mrs. Conclusion another chance to respond.
        mq.send(s)

        s, _ = mq.receive()
        s = s.decode()
        utils.say("Received %s" % s)

    # What I read must be the md5 of what I wrote or something's
    # gone wrong.
    what_i_sent = what_i_sent.encode()

    try:
        assert(s == hashlib.md5(what_i_sent).hexdigest())
    except AssertionError:
        raise AssertionError("Message corruption after %d iterations." % i)

    # MD5 the reply and write back to Mrs. Conclusion.
    s = hashlib.md5(s.encode()).hexdigest()
    utils.say("Sending %s" % s)
    mq.send(s)
    what_i_sent = s

utils.say("")
utils.say("%d iterations complete" % (i + 1))

utils.say("Destroying the message queue.")
mq.close()
# I could call simply mq.unlink() here but in order to demonstrate
# unlinking at the module level I'll do it that way.
posix_ipc.unlink_message_queue(params["MESSAGE_QUEUE_NAME"])



# Python imports
import unittest
from unittest import skipUnless
import datetime

# Project imports
import posix_ipc

from . import base as tests_base


# N_RELEASES is the number of times release() is called in test_release()
N_RELEASES = 1000000  # 1 million


class SemaphoreTestBase(tests_base.Base):
    """base class for Semaphore test classes"""
    def setUp(self):
        self.sem = posix_ipc.Semaphore(None, posix_ipc.O_CREX, initial_value=1)

    def tearDown(self):
        if self.sem:
            self.sem.unlink()

    def assertWriteToReadOnlyPropertyFails(self, property_name, value):
        """test that writing to a readonly property raises TypeError"""
        tests_base.Base.assertWriteToReadOnlyPropertyFails(self, self.sem, property_name, value)


class TestSemaphoreCreation(SemaphoreTestBase):
    """Exercise stuff related to creating Semaphores"""
    def test_no_flags(self):
        """tests that opening a semaphore with no flags opens the existing
        semaphore and doesn't create a new semaphore"""
        sem_copy = posix_ipc.Semaphore(self.sem.name)
        self.assertEqual(self.sem.name, sem_copy.name)

    def test_o_creat_existing(self):
        """tests posix_ipc.O_CREAT to open an existing semaphore without
        O_EXCL"""
        sem_copy = posix_ipc.Semaphore(self.sem.name, posix_ipc.O_CREAT)

        self.assertEqual(self.sem.name, sem_copy.name)

    def test_o_creat_new(self):
        """tests posix_ipc.O_CREAT to create a new semaphore without O_EXCL"""
        # I can't pass None for the name unless I also pass O_EXCL.
        name = tests_base.make_name()

        # Note: this method of finding an unused name is vulnerable to a race
        # condition. It's good enough for test, but don't copy it for use in
        # production code!
        name_is_available = False
        while not name_is_available:
            try:
                sem = posix_ipc.Semaphore(name)
                sem.close()
            except posix_ipc.ExistentialError:
                name_is_available = True
            else:
                name = tests_base.make_name()

        sem = posix_ipc.Semaphore(name, posix_ipc.O_CREAT)

        self.assertIsNotNone(sem)

        sem.unlink()

    @unittest.skipIf(tests_base.HAS_FREEBSD_BUG_206396, tests_base.FREEBSD_BUG_206396_SKIP_MSG)
    def test_o_excl(self):
        """tests O_CREAT | O_EXCL prevents opening an existing semaphore"""
        self.assertRaises(posix_ipc.ExistentialError, posix_ipc.Semaphore,
                          self.sem.name, posix_ipc.O_CREAT | posix_ipc.O_EXCL)

    @unittest.skipIf(tests_base.HAS_FREEBSD_BUG_206396, tests_base.FREEBSD_BUG_206396_SKIP_MSG)
    def test_o_crex(self):
        """tests O_CREX prevents opening an existing semaphore"""
        self.assertRaises(posix_ipc.ExistentialError, posix_ipc.Semaphore,
                          self.sem.name, posix_ipc.O_CREX)

    def test_randomly_generated_name(self):
        """tests that the randomly-generated name works"""
        # This is tested implicitly elsewhere but I want to test it explicitly
        sem = posix_ipc.Semaphore(None, posix_ipc.O_CREX)
        self.assertIsNotNone(sem.name)

        self.assertEqual(sem.name[0], '/')
        self.assertGreaterEqual(len(sem.name), 2)
        sem.unlink()

    def test_name_as_bytes(self):
        """Test that the name can be bytes."""
        name = bytes(tests_base.make_name(), 'ASCII')
        sem = posix_ipc.Semaphore(name, posix_ipc.O_CREX)
        self.assertEqual(name, bytes(sem.name, 'ASCII'))
        sem.unlink()
        sem.close()

    # don't bother testing mode, it's ignored by the OS?

    @skipUnless(posix_ipc.SEMAPHORE_VALUE_SUPPORTED, "Requires Semaphore.value support")
    def test_default_initial_value(self):
        """tests that the initial value is 0 by default"""
        sem = posix_ipc.Semaphore(None, posix_ipc.O_CREX)
        self.assertEqual(sem.value, 0)
        sem.unlink()

    @skipUnless(posix_ipc.SEMAPHORE_VALUE_SUPPORTED, "Requires Semaphore.value support")
    def test_zero_initial_value(self):
        """tests that the initial value is 0 when assigned"""
        sem = posix_ipc.Semaphore(None, posix_ipc.O_CREX, initial_value=0)
        self.assertEqual(sem.value, 0)
        sem.unlink()

    @skipUnless(posix_ipc.SEMAPHORE_VALUE_SUPPORTED, "Requires Semaphore.value support")
    def test_nonzero_initial_value(self):
        """tests that the initial value is non-zero when assigned"""
        sem = posix_ipc.Semaphore(None, posix_ipc.O_CREX, initial_value=42)
        self.assertEqual(sem.value, 42)
        sem.unlink()

    def test_kwargs(self):
        """ensure init accepts keyword args as advertised"""
        # mode 0x180 = 0600. Octal is difficult to express in Python 2/3 compatible code.
        sem = posix_ipc.Semaphore(None, flags=posix_ipc.O_CREX, mode=0x180, initial_value=0)
        sem.unlink()


class TestSemaphoreAquisitionAndRelease(SemaphoreTestBase):
    """Exercise acquiring & releasing semaphores"""
    def test_simple_acquisition(self):
        """tests that acquisition works"""
        self.sem.acquire()

    # test acquisition failures
    # def test_acquisition_no_timeout(self):
    # FIXME
    # This is hard to test since it should wait infinitely. Probably the way
    # to do it is to spawn another process that holds the semaphore for
    # maybe 10 seconds and have this process wait on it. That's complicated
    # and not a really great test.

    def test_acquisition_zero_timeout(self):
        """tests that acquisition w/timeout=0 implements non-blocking
        behavior"""
        # Should not raise an error
        self.sem.acquire(0)
        with self.assertRaises(posix_ipc.BusyError):
            self.sem.acquire(0)

    @skipUnless(posix_ipc.SEMAPHORE_TIMEOUT_SUPPORTED, "Requires Semaphore timeout support")
    def test_acquisition_nonzero_int_timeout(self):
        """tests that acquisition w/timeout=an int is reasonably accurate"""
        # Should not raise an error
        self.sem.acquire(0)

        # This should raise a busy error
        wait_time = 1
        start = datetime.datetime.now()
        with self.assertRaises(posix_ipc.BusyError):
            self.sem.acquire(wait_time)
        end = datetime.datetime.now()
        actual_delta = end - start
        expected_delta = datetime.timedelta(seconds=wait_time)

        delta = actual_delta - expected_delta

        self.assertEqual(delta.days, 0)
        self.assertEqual(delta.seconds, 0)
        # I don't want to test microseconds because that granularity
        # isn't under the control of this module.

    @skipUnless(posix_ipc.SEMAPHORE_TIMEOUT_SUPPORTED, "Requires Semaphore timeout support")
    def test_acquisition_nonzero_float_timeout(self):
        """tests that acquisition w/timeout=a float is reasonably accurate"""
        # Should not raise an error
        self.sem.acquire(0)

        # This should raise a busy error
        wait_time = 1.5
        start = datetime.datetime.now()
        with self.assertRaises(posix_ipc.BusyError):
            self.sem.acquire(wait_time)
        end = datetime.datetime.now()
        actual_delta = end - start
        expected_delta = datetime.timedelta(seconds=wait_time)

        delta = actual_delta - expected_delta

        self.assertEqual(delta.days, 0)
        self.assertEqual(delta.seconds, 0)
        # I don't want to test microseconds because that granularity
        # isn't under the control of this module.

    def test_release(self):
        """tests that release works"""
        # Not only does it work, I can do it as many times as I want! I had
        # tried some code that called release() SEMAPHORE_VALUE_MAX times, but
        # on platforms where that's ~2 billion, the test takes too long to run.
        # So I'll stick to a lower (but still very large) number of releases.
        n_releases = min(N_RELEASES, posix_ipc.SEMAPHORE_VALUE_MAX - 1)
        for i in range(n_releases):
            self.sem.release()

    def test_context_manager(self):
        """tests that context manager acquire/release works"""
        with self.sem as sem:
            if posix_ipc.SEMAPHORE_VALUE_SUPPORTED:
                self.assertEqual(sem.value, 0)
            with self.assertRaises(posix_ipc.BusyError):
                sem.acquire(0)

        if posix_ipc.SEMAPHORE_VALUE_SUPPORTED:
            self.assertEqual(sem.value, 1)

        # Should not raise an error.
        sem.acquire(0)


class TestSemaphoreDestruction(SemaphoreTestBase):
    def test_close_and_unlink(self):
        """tests that sem.close() and sem.unlink() work"""
        # sem.close() is hard to test since subsequent use of the semaphore
        # after sem.close() is undefined. All I can think of to do is call it
        # and note that it does not fail. Also, it allows sem.unlink() to
        # tell the OS to delete the semaphore entirely, so it makes sense
        # to test them together.
        self.sem.unlink()
        self.sem.close()
        self.assertRaises(posix_ipc.ExistentialError, posix_ipc.Semaphore,
                          self.sem.name)
        # Wipe this out so that self.tearDown() doesn't crash.
        self.sem = None


class TestSemaphorePropertiesAndAttributes(SemaphoreTestBase):
    def test_property_name(self):
        """exercise Semaphore.name"""
        self.assertGreaterEqual(len(self.sem.name), 2)

        self.assertEqual(self.sem.name[0], '/')

        self.assertWriteToReadOnlyPropertyFails('name', 'hello world')

    @skipUnless(posix_ipc.SEMAPHORE_VALUE_SUPPORTED, "Requires Semaphore.value support")
    def test_property_value(self):
        """exercise Semaphore.value if possible"""
        # test read, although this has been tested very thoroughly above
        self.assertEqual(self.sem.value, 1)

        self.assertWriteToReadOnlyPropertyFails('value', 42)


if __name__ == '__main__':
    unittest.main()



# Python imports
import unittest
import os
import resource

# Project imports
import posix_ipc

from . import base as tests_base

ONE_MILLION = 1000000


class TestModule(tests_base.Base):
    """Exercise the posix_ipc module-level functions and constants"""
    def test_constant_values(self):
        """test that constants are what I expect"""
        self.assertEqual(posix_ipc.O_CREAT, os.O_CREAT)
        self.assertEqual(posix_ipc.O_EXCL, os.O_EXCL)
        self.assertEqual(posix_ipc.O_CREX, posix_ipc.O_CREAT | posix_ipc.O_EXCL)
        self.assertEqual(posix_ipc.O_TRUNC, os.O_TRUNC)

        self.assertEqual(posix_ipc.PAGE_SIZE, resource.getpagesize())

        self.assertIn(posix_ipc.SEMAPHORE_TIMEOUT_SUPPORTED, (True, False))
        self.assertIn(posix_ipc.SEMAPHORE_VALUE_SUPPORTED, (True, False))

        self.assertGreaterEqual(posix_ipc.SEMAPHORE_VALUE_MAX, 1)

        self.assertIn(posix_ipc.MESSAGE_QUEUES_SUPPORTED, (True, False))

        if posix_ipc.MESSAGE_QUEUES_SUPPORTED:
            self.assertGreaterEqual(posix_ipc.QUEUE_MESSAGES_MAX_DEFAULT, 1)
            self.assertGreaterEqual(posix_ipc.QUEUE_MESSAGE_SIZE_MAX_DEFAULT, 1)
            self.assertGreaterEqual(posix_ipc.QUEUE_PRIORITY_MAX, 0)

        if hasattr(posix_ipc, 'USER_SIGNAL_MIN'):
            self.assertGreaterEqual(posix_ipc.USER_SIGNAL_MIN, 1)
        if hasattr(posix_ipc, 'USER_SIGNAL_MAX'):
            self.assertGreaterEqual(posix_ipc.USER_SIGNAL_MAX, 1)

        self.assertTrue(isinstance(posix_ipc.VERSION, str))

    def test_unlink_semaphore(self):
        """Exercise unlink_semaphore"""
        sem = posix_ipc.Semaphore(None, posix_ipc.O_CREX)
        posix_ipc.unlink_semaphore(sem.name)
        sem.close()
        self.assertRaises(posix_ipc.ExistentialError, posix_ipc.Semaphore,
                          sem.name)

    def test_unlink_shared_memory(self):
        """Exercise unlink_shared_memory"""
        mem = posix_ipc.SharedMemory(None, posix_ipc.O_CREX, size=1024)
        mem.close_fd()
        posix_ipc.unlink_shared_memory(mem.name)
        self.assertRaises(posix_ipc.ExistentialError, posix_ipc.SharedMemory,
                          mem.name)

    if posix_ipc.MESSAGE_QUEUES_SUPPORTED:
        def test_unlink_message_queue(self):
            """Exercise unlink_message_queue"""
            mq = posix_ipc.MessageQueue(None, posix_ipc.O_CREX)
            posix_ipc.unlink_message_queue(mq.name)
            mq.close()
            self.assertRaises(posix_ipc.ExistentialError,
                              posix_ipc.MessageQueue, mq.name)

        def test_constant_queue_priority_max(self):
            """Test that QUEUE_PRIORITY_MAX is reported correctly"""
            mq = posix_ipc.MessageQueue(None, posix_ipc.O_CREX)

            if posix_ipc.QUEUE_PRIORITY_MAX < ONE_MILLION:
                for sent_priority in range(posix_ipc.QUEUE_PRIORITY_MAX + 1):
                    mq.send('', priority=sent_priority)
                    msg, received_priority = mq.receive()
                    self.assertEqual(sent_priority, received_priority)
            # else:
                # QUEUE_PRIORITY_MAX is probably LONG_MAX or larger and
                # testing every value will take too long.

            self.assertRaises(ValueError, mq.send, '',
                              priority=posix_ipc.QUEUE_PRIORITY_MAX + 1)

            mq.unlink()
            mq.close()

    def test_errors(self):
        self.assertTrue(issubclass(posix_ipc.Error, Exception))
        self.assertTrue(issubclass(posix_ipc.SignalError, posix_ipc.Error))
        self.assertTrue(issubclass(posix_ipc.PermissionsError, posix_ipc.Error))
        self.assertTrue(issubclass(posix_ipc.ExistentialError, posix_ipc.Error))
        self.assertTrue(issubclass(posix_ipc.BusyError, posix_ipc.Error))


if __name__ == '__main__':
    unittest.main()



# Python imports
import unittest
from unittest import skipUnless
import time
import signal
import threading

# Project imports
import posix_ipc

from . import base as tests_base

if hasattr(posix_ipc, 'USER_SIGNAL_MIN'):
    # Due to Python bug http://bugs.python.org/issue20584, not all valid signal
    # values can be used. I noticed it on FreeBSD, it might be visible
    # elsewhere.
    if posix_ipc.USER_SIGNAL_MIN >= signal.NSIG:
        SIGNAL_VALUE = signal.SIGHUP
    else:
        SIGNAL_VALUE = posix_ipc.USER_SIGNAL_MIN
else:
    SIGNAL_VALUE = signal.SIGHUP

signal_handler_value_received = 0


def signal_handler(signal_value, frame):
    """Handle signal sent for msg q notification test."""
    global signal_handler_value_received
    signal_handler_value_received = signal_value


def threaded_notification_handler_one_shot(test_case_instance):
    """Handle msg q notification in a thread without rearming notification."""
    test_case_instance.threaded_notification_called = True
    test_case_instance.notification_event.set()


def threaded_notification_handler_rearm(test_case_instance):
    """Handle msg q notification in a thread and rearm notification."""
    # Rearm.
    param = (threaded_notification_handler_rearm, test_case_instance)
    test_case_instance.mq.request_notification(param)

    test_case_instance.threaded_notification_called = True
    test_case_instance.notification_event.set()


class MessageQueueTestBase(tests_base.Base):
    """base class for MessageQueue test classes"""
    def setUp(self):
        # Create a small queue to reduce the likelihood of resource exhaustion
        # See https://github.com/osvenskan/posix_ipc/issues/42
        self.mq = posix_ipc.MessageQueue(None, posix_ipc.O_CREX,
                                         max_messages=10,
                                         max_message_size=10)

    def tearDown(self):
        if self.mq:
            self.mq.close()
            self.mq.unlink()

    def assertWriteToReadOnlyPropertyFails(self, property_name, value):
        """test that writing to a readonly property raises TypeError"""
        tests_base.Base.assertWriteToReadOnlyPropertyFails(self, self.mq, property_name, value)


@skipUnless(posix_ipc.MESSAGE_QUEUES_SUPPORTED, "Requires MessageQueue support")
class TestMessageQueueCreation(MessageQueueTestBase):
    """Exercise stuff related to creating MessageQueue"""
    def test_no_flags(self):
        """tests that opening a queue with no flags opens the existing
        queue and doesn't create a new queue"""
        mq_copy = posix_ipc.MessageQueue(self.mq.name)
        self.assertEqual(self.mq.name, mq_copy.name)
        mq_copy.close()

    def test_o_creat_existing(self):
        """tests posix_ipc.O_CREAT to open an existing MessageQueue without
        O_EXCL"""
        mq_copy = posix_ipc.MessageQueue(self.mq.name, posix_ipc.O_CREAT)
        self.assertEqual(self.mq.name, mq_copy.name)
        mq_copy.close()

    def test_o_creat_new(self):
        """tests posix_ipc.O_CREAT to create a new MessageQueue without
        O_EXCL"""
        # I can't pass None for the name unless I also pass O_EXCL.
        name = tests_base.make_name()

        # Note: this method of finding an unused name is vulnerable to a
        # race condition. It's good enough for test, but don't copy it for
        # use in production code!
        name_is_available = False
        while not name_is_available:
            try:
                mq = posix_ipc.MessageQueue(name)
                mq.close()
            except posix_ipc.ExistentialError:
                name_is_available = True
            else:
                name = tests_base.make_name()

        mq = posix_ipc.MessageQueue(name, posix_ipc.O_CREAT)
        self.assertIsNotNone(mq)
        mq.close()
        mq.unlink()

    def test_o_crex(self):
        """tests O_CREX prevents opening an existing MessageQueue"""
        self.assertRaises(posix_ipc.ExistentialError, posix_ipc.MessageQueue,
                          self.mq.name, posix_ipc.O_CREX)

    def test_randomly_generated_name(self):
        """tests that the randomly-generated name works"""
        # This is tested implicitly elsewhere but I want an explicit test
        mq = posix_ipc.MessageQueue(None, posix_ipc.O_CREX)
        self.assertIsNotNone(mq.name)

        self.assertEqual(mq.name[0], '/')
        self.assertGreaterEqual(len(mq.name), 2)
        mq.close()
        mq.unlink()

    def test_name_as_bytes(self):
        """Test that the name can be bytes."""
        name = bytes(tests_base.make_name(), 'ASCII')
        mq = posix_ipc.MessageQueue(name, posix_ipc.O_CREX)
        self.assertEqual(name, bytes(mq.name, 'ASCII'))
        mq.close()
        mq.unlink()

    # don't bother testing mode, it's ignored by the OS?

    def test_max_messages(self):
        """test that the max_messages param is respected"""
        mq = posix_ipc.MessageQueue(None, posix_ipc.O_CREX, max_messages=1)
        mq.send('foo')
        self.assertRaises(posix_ipc.BusyError, mq.send, 'bar', timeout=0)
        mq.close()
        mq.unlink()

    def test_max_message_size(self):
        """test that the max_message_size param is respected"""
        mq = posix_ipc.MessageQueue(None, posix_ipc.O_CREX,
                                    max_message_size=10)
        self.assertRaises(ValueError, mq.send, ' ' * 11)
        mq.close()
        mq.unlink()

    def test_read_flag_new_queue(self):
        """test that the read flag is respected on a new queue"""
        mq = posix_ipc.MessageQueue(None, posix_ipc.O_CREX, read=False)
        mq.send('foo')
        self.assertRaises(posix_ipc.PermissionsError, mq.receive)
        mq.close()
        mq.unlink()

    def test_read_flag_existing_queue(self):
        """test that the read flag is respected on an existing queue"""
        mq = posix_ipc.MessageQueue(self.mq.name, read=False)
        mq.send('foo')
        self.assertRaises(posix_ipc.PermissionsError, mq.receive)
        mq.close()

    def test_write_flag_new_queue(self):
        """test that the write flag is respected on a new queue"""
        mq = posix_ipc.MessageQueue(None, posix_ipc.O_CREX, write=False)
        self.assertRaises(posix_ipc.PermissionsError, mq.send, 'foo')
        mq.close()
        mq.unlink()

    def test_write_flag_existing_queue(self):
        """test that the write flag is respected on an existing queue"""
        mq = posix_ipc.MessageQueue(self.mq.name, write=False)
        self.assertRaises(posix_ipc.PermissionsError, mq.send, 'foo')
        mq.close()

    def test_kwargs(self):
        """ensure init accepts keyword args as advertised"""
        mq = posix_ipc.MessageQueue(None, flags=posix_ipc.O_CREX, mode=0o0600, max_messages=1,
                                    max_message_size=256, read=True, write=True)
        mq.close()
        mq.unlink()


@skipUnless(posix_ipc.MESSAGE_QUEUES_SUPPORTED, "Requires MessageQueue support")
class TestMessageQueueSendReceive(MessageQueueTestBase):
    """Exercise send() and receive()"""
    def test_send(self):
        """Test that simple send works.

        It's already tested elsewhere implicitly, but I want an explicit
        test.
        """
        self.mq.send('foo')

    # FIXME how to test that send with timeout=None waits as expected?

    def test_send_timeout_keyword(self):
        """Test that the timeout keyword of send works"""
        n_msgs = self.mq.max_messages
        while n_msgs:
            self.mq.send(' ')
            n_msgs -= 1

        self.assertRaises(posix_ipc.BusyError, self.mq.send, 'foo', timeout=0)

    def test_send_timeout_positional(self):
        """Test that the timeout positional param of send works"""
        n_msgs = self.mq.max_messages
        while n_msgs:
            self.mq.send(' ')
            n_msgs -= 1

        self.assertRaises(posix_ipc.BusyError, self.mq.send, 'foo', 0)

    def test_send_timeout_zero_success(self):
        """Test that send w/a timeout=0 succeeds if queue is not full."""
        self.mq.send('foo', timeout=0)

    def test_send_timeout_zero_fails(self):
        """Test that send w/a timeout=0 raises BusyError if queue is
        full.
        """
        n_msgs = self.mq.max_messages
        while n_msgs:
            self.mq.send(' ')
            n_msgs -= 1

        self.assertRaises(posix_ipc.BusyError, self.mq.send, 'foo',
                          timeout=0)

    def test_send_nonzero_timeout(self):
        """Test that a non-zero timeout to send is respected."""
        n_msgs = self.mq.max_messages
        while n_msgs:
            self.mq.send(' ')
            n_msgs -= 1

        start = time.time()
        self.assertRaises(posix_ipc.BusyError, self.mq.send, ' ',
                          timeout=1.0)
        elapsed = time.time() - start
        # I don't insist on extreme precision.
        self.assertTrue(elapsed >= 1.0)
        self.assertTrue(elapsed < 1.5)

    def test_send_priority_default(self):
        """Test that the send priority defaults to 0"""
        self.mq.send('foo')
        self.assertEqual(self.mq.receive(), ('foo'.encode(), 0))

    def test_send_fifo_default(self):
        """Test that the queue order is FIFO by default"""
        alphabet = 'abcdefg'
        for c in alphabet:
            self.mq.send(c)

        for c in alphabet:
            self.assertEqual(self.mq.receive(), (c.encode(), 0))

    def test_send_priority(self):
        """Test that the priority param is respected"""
        # By simple FIFO, these would be returned lowest, highest, middle.
        # Instead they'll be returned highest, middle, lowest
        self.mq.send('lowest', priority=1)
        self.mq.send('highest', priority=3)
        self.mq.send('middle', priority=2)

        self.assertEqual(self.mq.receive(), ('highest'.encode(), 3))
        self.assertEqual(self.mq.receive(), ('middle'.encode(), 2))
        self.assertEqual(self.mq.receive(), ('lowest'.encode(), 1))

    def test_send_priority_keyword(self):
        """Test that the priority keyword of send works"""
        self.mq.send('foo', priority=42)
        self.assertEqual(self.mq.receive(), ('foo'.encode(), 42))

    def test_send_priority_positional(self):
        """Test that the priority positional param of send works"""
        self.mq.send('foo', 0, 42)
        self.assertEqual(self.mq.receive(), ('foo'.encode(), 42))

    def test_send_kwargs(self):
        """ensure send() accepts keyword args as advertised"""
        self.mq.send('foo', timeout=0, priority=0)

    # ##### test receive()

    def test_receive(self):
        """Test that simple receive works.

        It's already tested elsewhere implicitly, but I want an explicit
        test.
        """
        self.mq.send('foo', priority=3)

        self.assertEqual(self.mq.receive(), ('foo'.encode(), 3))

    def test_receive_timeout_positional(self):
        """Test that the timeout positional param of receive works."""
        self.assertRaises(posix_ipc.BusyError, self.mq.receive, 0)

    def test_receive_nonzero_timeout(self):
        """Test that a non-zero timeout to receive is respected."""
        start = time.time()
        self.assertRaises(posix_ipc.BusyError, self.mq.receive, 1.0)
        elapsed = time.time() - start
        # I don't insist on extreme precision.
        self.assertTrue(elapsed >= 1.0)
        self.assertTrue(elapsed < 1.5)

    # FIXME how to test that timeout=None waits forever?

    def test_receive_max_length(self):
        '''Test that messages of maximum length are handled correctly.'''
        # Ensure the max message size is what I think it is.
        assert self.mq.max_message_size == 10

        self.mq.send('abcdefghij')
        msg, _ = self.mq.receive()
        assert msg == b'abcdefghij'

@skipUnless(posix_ipc.MESSAGE_QUEUES_SUPPORTED, "Requires MessageQueue support")
class TestMessageQueueNotification(MessageQueueTestBase):
    """exercise request_notification()"""
    def test_request_notification_signal(self):
        """Exercise notification by signal"""
        global someone_rang_the_doorbell

        self.mq.request_notification(SIGNAL_VALUE)

        signal.signal(SIGNAL_VALUE, signal_handler)

        someone_rang_the_doorbell = False

        self.mq.send('')

        self.assertEqual(signal_handler_value_received, SIGNAL_VALUE)

    def test_request_notification_signal_one_shot(self):
        """Test that after notification by signal, notification is
        cancelled"""
        global signal_handler_value_received

        self.mq.request_notification(SIGNAL_VALUE)

        signal.signal(SIGNAL_VALUE, signal_handler)

        signal_handler_value_received = 0

        self.mq.send('')

        self.assertEqual(signal_handler_value_received, SIGNAL_VALUE)

        # Reset the global flag
        signal_handler_value_received = 0

        self.mq.send('')

        # Flag should not be set because it's only supposed to fire
        # notification when the queue changes from empty to non-empty,
        # and there was already 1 msg in the Q when I called send() above.
        self.assertEqual(signal_handler_value_received, 0)

        # empty the queue
        self.mq.receive()
        self.mq.receive()

        self.assertEqual(signal_handler_value_received, 0)

        self.mq.send('')

        # Flag should still not be set because notification was cancelled
        # after it fired the first time.
        self.assertEqual(signal_handler_value_received, 0)

    def test_request_notification_cancel_default(self):
        """Test that notification can be cancelled with the default param"""
        global signal_handler_value_received

        self.mq.request_notification(SIGNAL_VALUE)

        signal.signal(SIGNAL_VALUE, signal_handler)

        signal_handler_value_received = 0

        # Cancel notification
        self.mq.request_notification()

        self.mq.send('')

        self.assertEqual(signal_handler_value_received, 0)

    def test_request_notification_cancel_multiple(self):
        """Test that notification can be cancelled multiple times"""
        self.mq.request_notification(SIGNAL_VALUE)

        # Cancel notification lots of times
        for i in range(1000):
            self.mq.request_notification()

    def test_request_notification_threaded_one_shot(self):
        """Test simple threaded notification"""

        self.threaded_notification_called = False

        param = (threaded_notification_handler_one_shot, self)
        self.mq.request_notification(param)

        self.notification_event = threading.Event()

        self.mq.send('')

        # I have to wait on a shared event to ensure that the notification
        # handler's thread gets a chance to execute before I make my
        # assertion.
        self.notification_event.wait(5)

        self.assertTrue(self.threaded_notification_called)

    def test_request_notification_threaded_rearm(self):
        """Test threaded notification in which the notified thread rearms
        the notification"""

        self.threaded_notification_called = False

        param = (threaded_notification_handler_rearm, self)
        self.mq.request_notification(param)

        self.notification_event = threading.Event()

        # Re-arm several times.
        for i in range(10):
            self.mq.send('')

            # I have to wait on a shared event to ensure that the
            # notification handler's thread gets a chance to execute before
            # I make my assertion.
            self.notification_event.wait(5)

            self.assertTrue(self.threaded_notification_called)

            self.mq.receive()

            self.notification_event.clear()


@skipUnless(posix_ipc.MESSAGE_QUEUES_SUPPORTED, "Requires MessageQueue support")
class TestMessageQueueDestruction(MessageQueueTestBase):
    """exercise close() and unlink()"""

    def test_close_and_unlink(self):
        """tests that mq.close() and mq.unlink() work"""
        # mq.close() is hard to test since subsequent use of the queue
        # after mq.close() is undefined. All I can think of to do is call it
        # and note that it does not fail. Also, it allows mq.unlink() to
        # tell the OS to delete the queue entirely, so it makes sense
        # to test them together,
        self.mq.unlink()
        self.mq.close()
        self.assertRaises(posix_ipc.ExistentialError, posix_ipc.MessageQueue,
                          self.mq.name)

        # Wipe this out so that self.tearDown() doesn't crash.
        self.mq = None


@skipUnless(posix_ipc.MESSAGE_QUEUES_SUPPORTED, "Requires MessageQueue support")
class TestMessageQueuePropertiesAndAttributes(MessageQueueTestBase):
    """Exercise props and attrs"""
    def test_property_name(self):
        """exercise MessageQueue.name"""
        self.assertGreaterEqual(len(self.mq.name), 2)

        self.assertEqual(self.mq.name[0], '/')

        self.assertWriteToReadOnlyPropertyFails('name', 'hello world')

    def test_property_mqd(self):
        """exercise MessageQueue.mqd"""
        self.assertNotEqual(self.mq.mqd, -1)

        # The mqd is of type mqd_t. I can't find doc that states what this
        # type is. All I know is that -1 is an error so it's probably
        # int-ish, but I can't tell exactly what to expect.
        self.assertWriteToReadOnlyPropertyFails('mqd', 42)

    def test_fileno(self):
        """exercise MessageQueue.fileno()"""
        self.assertEqual(self.mq.mqd, self.mq.fileno())

    def test_property_max_messages(self):
        """exercise MessageQueue.max_messages"""
        self.assertGreaterEqual(self.mq.max_messages, 0)

        self.assertWriteToReadOnlyPropertyFails('max_messages', 42)

    def test_property_max_message_size(self):
        """exercise MessageQueue.max_message_size"""
        self.assertGreaterEqual(self.mq.max_message_size, 0)

        self.assertWriteToReadOnlyPropertyFails('max_message_size', 42)

    def test_property_current_messages(self):
        """exercise MessageQueue.current_messages"""
        self.assertEqual(self.mq.current_messages, 0)

        self.mq.send('')
        self.assertEqual(self.mq.current_messages, 1)
        self.mq.send('')
        self.mq.send('')
        self.mq.send('')
        self.assertEqual(self.mq.current_messages, 4)
        self.mq.receive()
        self.assertEqual(self.mq.current_messages, 3)
        self.mq.receive()
        self.assertEqual(self.mq.current_messages, 2)
        self.mq.receive()
        self.assertEqual(self.mq.current_messages, 1)
        self.mq.receive()
        self.assertEqual(self.mq.current_messages, 0)

        self.assertWriteToReadOnlyPropertyFails('current_messages', 42)

    def test_block_flag_default_value_and_writability(self):
        """test that the block flag is True by default and can be changed"""
        mq = posix_ipc.MessageQueue(None, posix_ipc.O_CREX)

        self.assertTrue(mq.block)

        # Toggle. I expect no errors. It's good to test this on a queue
        # that's only used for this particular test. I would rather have
        # all the other tests execute using the default value of block
        # on self.mq.
        mq.block = False
        mq.block = True

        mq.close()
        mq.unlink()

    def test_block_flag_false(self):
        """test blocking behavior when flag is false"""
        mq = posix_ipc.MessageQueue(None, posix_ipc.O_CREX, max_messages=3)

        mq.block = False

        # Queue is empty, so receive() should immediately raise BusyError
        start = time.time()
        self.assertRaises(posix_ipc.BusyError, mq.receive, 10)
        elapsed = time.time() - start

        # Not only should receive() have raised BusyError, it should have
        # done so "immediately". I don't insist on extreme precision since
        # OS-level task switching might mean that elapsed time is not
        # vanishingly small as one might expect under most circumstances.
        self.assertTrue(elapsed < 1.0)

        # Now test send() the same way.
        mq.send(' ')
        mq.send(' ')
        mq.send(' ')

        # Queue is now full.
        start = time.time()
        self.assertRaises(posix_ipc.BusyError, mq.send, ' ', 10)
        elapsed = time.time() - start
        self.assertTrue(elapsed < 1.0)

        mq.close()
        mq.unlink()


if __name__ == '__main__':
    unittest.main()



import posix_ipc
import selectors

# This program uses `posix_ipc` together with the `selectors`library from the
# Python standard library. `selectors` provides "high-level I/O multiplexing" akin to having an
# event library.

# The message queue is created as usual
mq = posix_ipc.MessageQueue("/python_ipc_test", flags=posix_ipc.O_CREAT)
mq.block = False

# Function is defined to handle events on the queue

def accept(message_queue, mask):
    (msg, prio) = message_queue.receive()
    print("Message: ", msg)
    print("Priority: ", prio)

# The selector can now be created...

sel = selectors.DefaultSelector()

# ... and the message queue is registered. Other event sources could also be
# registered simultaneously, but for now we stick to the queue

sel.register(mq, selectors.EVENT_READ, accept)

# `.select()` will block until an event is triggered

print("Listening...")

events = sel.select()
for key, mask in events:
    # `.data` contains the third argument from `.register` above -- we use it for the callback.
    callback = key.data
    callback(key.fileobj, mask)

# With the message successfully received, we can unlink and close.

mq.unlink()
mq.close()



# Python imports
import numbers
import platform
import unittest
import mmap
import os
import sys

# Project imports
import posix_ipc

from . import base as tests_base

_IS_MACOS = "Darwin" in platform.uname()


class TestMemory(tests_base.Base):
    """Exercise the SharedMemory class"""
    # SIZE should be something that's not a power of 2 since that's more
    # likely to expose odd behavior.
    SIZE = 3333

    def setUp(self):
        self.mem = posix_ipc.SharedMemory('/foo', posix_ipc.O_CREX,
                                          size=self.SIZE)

    def tearDown(self):
        if self.mem:
            self.mem.close_fd()
            self.mem.unlink()

    def assertWriteToReadOnlyPropertyFails(self, property_name, value):
        """test that writing to a readonly property raises TypeError"""
        tests_base.Base.assertWriteToReadOnlyPropertyFails(self, self.mem,
                                                           property_name, value)

    def test_ctor_no_flags_existing(self):
        """tests that opening a memory segment with no flags opens the existing
        memory and doesn't create a new segment"""
        mem_copy = posix_ipc.SharedMemory('/foo')
        self.assertEqual(self.mem.name, mem_copy.name)

    def test_ctor_no_flags_non_existent(self):
        """test that attempting to open a non-existent memory segment with no
        flags fails"""
        self.assertRaises(posix_ipc.ExistentialError, posix_ipc.SharedMemory,
                          '/fjksfjkhsdakh')

    def test_ctor_o_creat_existing(self):
        """tests posix_ipc.O_CREAT to open an existing segment without O_EXCL"""
        mem_copy = posix_ipc.SharedMemory(self.mem.name, posix_ipc.O_CREAT)

        self.assertEqual(self.mem.name, mem_copy.name)

    def test_o_creat_new(self):
        """tests posix_ipc.O_CREAT to create a new mem segment without O_EXCL"""
        mem = posix_ipc.SharedMemory('/lsdhfkjahdskjf', posix_ipc.O_CREAT,
                                     size=4096)
        self.assertIsNotNone(mem)
        mem.close_fd()
        mem.unlink()

    def test_o_excl(self):
        """tests O_CREAT | O_EXCL prevents opening an existing memory segment"""
        self.assertRaises(posix_ipc.ExistentialError, posix_ipc.SharedMemory,
                          '/foo', posix_ipc.O_CREAT | posix_ipc.O_EXCL)

    def test_o_crex(self):
        """tests O_CREX prevents opening an existing memory segment"""
        self.assertRaises(posix_ipc.ExistentialError, posix_ipc.SharedMemory,
                          '/foo', posix_ipc.O_CREX)

    @unittest.skipIf(_IS_MACOS, "O_TRUNC is not supported under macOS")
    def test_o_trunc(self):
        """Test that O_TRUNC truncates the memory to 0 bytes"""
        mem_copy = posix_ipc.SharedMemory(self.mem.name, posix_ipc.O_TRUNC)

        self.assertEqual(mem_copy.size, 0)

    def test_randomly_generated_name(self):
        """tests that the randomly-generated name works"""
        mem = posix_ipc.SharedMemory(None, posix_ipc.O_CREX, size=1024)
        self.assertIsNotNone(mem.name)
        self.assertEqual(mem.name[0], '/')
        self.assertGreaterEqual(len(mem.name), 2)
        mem.close_fd()
        mem.unlink()

    def test_name_as_bytes(self):
        """Test that the name can be bytes."""
        name = bytes(tests_base.make_name(), 'ASCII')
        mem = posix_ipc.SharedMemory(name, posix_ipc.O_CREX, size=4096)
        self.assertEqual(name, bytes(mem.name, 'ASCII'))
        mem.close_fd()
        mem.unlink()

    # # don't bother testing mode, it's ignored by the OS?

    def test_mmap_size(self):
        """test that the size specified is (somewhat) respected by mmap()"""
        # In limited testing, Linux respects the exact size specified in the
        # SharedMemory() ctor when creating the mmapped file.
        # e.g. when self.SIZE = 3333, the
        # mmapped file is also 3333 bytes.
        #
        # macOS's mmapped files always have sizes that are mod mmap.PAGESIZE.
        #
        # I haven't tested other operating systems.
        #
        # AFAICT the specification doesn't demand that the size has to match
        # exactly, so this code accepts either value as correct.

        delta = self.SIZE % mmap.PAGESIZE

        if delta:
            # Round up to nearest block size
            crude_size = (self.SIZE - delta) + mmap.PAGESIZE
        else:
            crude_size = self.SIZE

        self.assertIn(self.mem.size, (self.SIZE, crude_size))

        f = mmap.mmap(self.mem.fd, self.SIZE)

        self.assertIn(f.size(), (self.SIZE, crude_size))

        f.close()

    def test_ctor_read_only_flag(self):
        """test that specifying the readonly flag prevents writing"""
        mem = posix_ipc.SharedMemory(self.mem.name, read_only=True)
        f = mmap.mmap(mem.fd, self.mem.size, prot=mmap.PROT_READ)
        self.assertRaises(TypeError, f.write, 'hello world')
        mem.close_fd()

    @unittest.skipUnless(sys.stdin.fileno() == 0, "Requires stdin to have file number 0")
    def test_ctor_fd_can_become_zero(self):
        """test that SharedMemory accepts 0 as valid file descriptor"""
        # ref: https://github.com/osvenskan/posix_ipc/issues/2
        # This test relies on OS compliance with the POSIX spec. Specifically, the spec for
        # shm_open() says --
        #
        #     shm_open() shall return a file descriptor for the shared memory
        #     object that is the lowest numbered file descriptor not currently
        #     open for that process.
        #
        # ref: http://pubs.opengroup.org/onlinepubs/009695399/functions/shm_open.html
        #
        # So, on systems compliant with that particular part of the spec, if I open a SharedMemory
        # segment after closing stdin (which has fd == 0), the SharedMemory segment, should be
        # assigned fd 0.
        os.close(0)

        # I have to supply a size here, otherwise the call to close_fd() will fail under macOS.
        # See here for another report of the same behavior:
        # https://stackoverflow.com/questions/35371133/close-on-shared-memory-in-osx-causes-invalid-argument-error
        mem = posix_ipc.SharedMemory(None, posix_ipc.O_CREX, size=4096)
        mem_fd = mem.fd
        # Clean up before attempting the assertion in case the assertion fails.
        mem.close_fd()
        mem.unlink()

        self.assertEqual(mem_fd, 0)

    def test_object_method_close_fd(self):
        """test that SharedMemory.close_fd() closes the file descriptor"""
        mem = posix_ipc.SharedMemory(self.mem.name)

        mem.close_fd()

        self.assertEqual(mem.fd, -1)

        # On at least one platform (my Mac), this raises OSError under Python 2.7 and ValueError
        # under Python 3.6.
        with self.assertRaises((OSError, ValueError)):
            os.fdopen(mem.fd)

    def test_unlink(self):
        """test that SharedMemory.unlink() deletes the segment"""
        name = self.mem.name
        self.mem.close_fd()
        self.mem.unlink()
        self.assertRaises(posix_ipc.ExistentialError, getattr,
                          self.mem, 'size')
        self.assertRaises(posix_ipc.ExistentialError, posix_ipc.SharedMemory,
                          name)
        self.mem = None

    def test_name_property(self):
        """exercise SharedMemory.name"""
        self.assertGreaterEqual(len(self.mem.name), 2)

        self.assertEqual(self.mem.name[0], '/')

        self.assertWriteToReadOnlyPropertyFails('name', 'hello world')

    def test_fd_property(self):
        """exercise SharedMemory.fd"""
        self.assertIsInstance(self.mem.fd, numbers.Integral)

        self.assertWriteToReadOnlyPropertyFails('fd', 42)

    def test_fileno(self):
        """exercise SharedMemory.fileno"""
        self.assertEqual(self.mem.fd, self.mem.fileno())

    def test_size_property(self):
        """exercise SharedMemory.size"""
        self.assertIsInstance(self.mem.size, numbers.Integral)

        self.assertWriteToReadOnlyPropertyFails('size', 42)


class TestMemoryResize(tests_base.Base):
    """Exercise various aspects of resizing an existing SharedMemory segment.

    The more interesting aspects of this test don't run under macOS because resizing isn't
    supported on that platform.
    """
    def setUp(self):
        # In constrast to the other memory test that deliberately uses an odd size, this test
        # uses a product of the system's block size. As noted above, the spec doesn't require
        # the segment to exactly respect the specified size. In practice probably all systems
        # respect it as long as it's evenly divisible by the block size.
        # One of the tests in this case attempts to cut the memory size in half, and I want to
        # mitigate the possibility that it will fail due to a platform-specific implementation
        # (e.g. trying to create a segment that's smaller than some arbitrary OS minimum).
        # Creating a segment that's twice the size of the block size seems the best option since
        # dividing it in half still leaves a segment size that's likely to be acceptable on
        # all platforms.
        self.original_size = mmap.PAGESIZE * 2
        self.mem = posix_ipc.SharedMemory(None, posix_ipc.O_CREX, size=self.original_size)

    def tearDown(self):
        if self.mem:
            self.mem.close_fd()
            self.mem.unlink()

    def test_ctor_second_handle_default_size_no_change(self):
        """opening an existing segment with the default size shouldn't change the size."""
        mem = posix_ipc.SharedMemory(self.mem.name)
        self.assertEqual(mem.size, self.original_size)
        self.assertEqual(self.mem.size, self.original_size)
        mem.close_fd()

    def test_ctor_second_handle_explicit_size_no_change(self):
        """opening an existing segment with an explicit size of 0 shouldn't change the size."""
        mem = posix_ipc.SharedMemory(self.mem.name, size=0)
        self.assertEqual(mem.size, self.original_size)
        self.assertEqual(self.mem.size, self.original_size)
        mem.close_fd()

    @unittest.skipIf(_IS_MACOS, "Changing shared memory size is not supported under macOS")
    def test_ctor_second_handle_size_increase(self):
        """exercise increasing the size of an existing segment via a second handle to it"""
        new_size = self.original_size * 2
        mem = posix_ipc.SharedMemory(self.mem.name, size=new_size)
        self.assertEqual(mem.size, new_size)
        self.assertEqual(self.mem.size, new_size)
        mem.close_fd()

    @unittest.skipIf(_IS_MACOS, "Changing shared memory size is not supported under macOS")
    def test_ctor_second_handle_size_decrease(self):
        """exercise decreasing the size of an existing segment via a second handle to it"""
        new_size = self.original_size // 2
        mem = posix_ipc.SharedMemory(self.mem.name, size=new_size)
        self.assertEqual(mem.size, new_size)
        self.assertEqual(self.mem.size, new_size)
        mem.close_fd()

    def test_ftruncate_increase(self):
        """exercise increasing the size of an existing segment from 0 via ftruncate()"""
        mem = posix_ipc.SharedMemory(None, posix_ipc.O_CREX)
        self.assertEqual(mem.size, 0)
        new_size = mmap.PAGESIZE
        os.ftruncate(mem.fd, new_size)
        self.assertEqual(mem.size, new_size)


if __name__ == '__main__':
    unittest.main()



import subprocess
import platform
import os
import shlex

# Set these to None for compile/link debugging or subprocess.PIPE to silence
# compiler warnings and errors.
STDOUT = subprocess.PIPE
STDERR = subprocess.PIPE
# STDOUT = None
# STDERR = None


# OUTPUT_FILEPATH is the file that this script will write, if necessary. The path is relative
# to the project root. Setuptools guarantees that the project root will be the current working
# directory when this script executes.
OUTPUT_FILEPATH = "./src/system_info.h"


# A few behaviors depend on whether or not this runs on a Mac.
IS_MAC = ("Darwin" in platform.uname())


class DiscoveryError(Exception):
    '''Exception raised when this script is unable to discover a value that it needs.'''
    pass


class POSIXNonComplianceWarning(UserWarning):
    '''Warning emitted when the underlying OS appears not to be POSIX compliant. POSIX compliance
    ensures (among other things) that important system information is available via sysconf().
    See https://github.com/osvenskan/posix_ipc/issues/81 for details.
    '''
    pass


def does_build_succeed(filename, linker_options=""):
    '''Returns True if the file compiles and links successfully, False otherwise.

    It can be perfectly normal for the build to fail, e.g. when building sniff_sem_getvalue.c on
    a system where sem_getvalue() doesn't exist.

    Note that unlike compile_and_run(), this just builds an executable. It does not attempt to
    run that executable.
    '''
    # There are two things to note about the command --
    #   - If there's a linker option like -lrt, it needs to come *after*
    #     the specification of the C file or linking will fail on Ubuntu 11.10
    #     (maybe because of the gcc version?)
    #   - Some versions of Linux place the sem_xxx() functions in libpthread.
    #     Rather than testing whether or not it's needed, I just specify it
    #     everywhere since it's harmless to specify it when it's not needed.
    cc = os.getenv("CC", "cc")
    cmd = [
           *shlex.split(cc),
           '-Wall',
           '-o',
           f'./build_support/src/{filename[:-2]}',
           f'./build_support/src/{filename}',
           linker_options,
           '-lpthread'
           ]
    p = subprocess.Popen(cmd, stdout=STDOUT, stderr=STDERR)

    # p.wait() returns the process' return code, so 0 implies that
    # the compile & link succeeded.
    return not bool(p.wait())


def compile_and_run(filename, linker_options=""):
    '''Compiles and links the file, runs the executable, and returns whatever the executable
    prints to stdout.

    Failure of any of the steps (compile, link, run) is unexepected. This function returns None
    in that case.
    '''
    if does_build_succeed(filename, linker_options):
        cmd = f"./build_support/src/{filename[:-2]}"
        try:
            s = subprocess.Popen([cmd], stdout=subprocess.PIPE).communicate()[0]
            return s.strip().decode()
        except Exception:
            # Execution resulted in an error. This is unexpected.
            return None
    else:
        # Build resulted in an error. This is unexpected.
         return None


def get_sysctl_value(name):
    """Given a sysctl name (e.g. 'kern.mqueue.maxmsg'), returns sysctl's value for that variable
    or None if the sysctl call fails (unknown name, sysctl not supported, etc.)
    """
    s = None
    try:
        # I redirect stderr to /dev/null because if sysctl is availble but
        # doesn't know about the particular item I'm querying, it will
        # kvetch with a message like 'second level name mqueue in
        # kern.mqueue.maxmsg is invalid'. This always happens under OS X
        # (which doesn't have any kern.mqueue values) and under FreeBSD when
        # the mqueuefs kernel module isn't loaded.
        s = subprocess.Popen(["sysctl", "-n", name],
                             stdout=subprocess.PIPE,
                             stderr=open(os.devnull, 'rw')).communicate()[0]
        s = s.strip().decode()
    except Exception:
        pass

    return s


def maybe_get_sysconf_value(name, complain_if_not_present=False):
    """Returns the value of a sysconf entry (e.g. 'SC_PAGESIZE') if the entry exists. If the value
    isn't available via sysconf, this function returns None.

    When complain_if_not_present is True, that means the sysconf value must be present for the
    system to be POSIX compliant. If the value is required but not present, this function raises
    a POSIXNonComplianceWarning to alert the user.

    Reference: https://pubs.opengroup.org/onlinepubs/9799919799.2024edition/functions/sysconf.html
    """
    value = None

    # Years ago, Cygwin didn't support os.sysconf_names. That has probably changed, but I don't
    # want to assume.
    if hasattr(os, 'sysconf_names'):
        if name in os.sysconf_names:
            value = os.sysconf(name)

    if complain_if_not_present and (value is None):
        raise POSIXNonComplianceWarning(f'Value "{name}" is missing from sysconf')

    return value


def sniff_realtime_lib():
    rc = None
    filename = "sniff_realtime_lib.c"

    if does_build_succeed(filename):
        # Realtime libs not needed
        rc = False
    else:
        # cc failed when not linked to realtime libs; let's try again
        # with the realtime libs involved and see if things go better.
        if does_build_succeed(filename, "-lrt"):
            # Realtime libs are needed
            rc = True

    if rc is None:
        raise DiscoveryError('Unable to determine whether realtime lib is needed to build')

    return rc


def sniff_sem_getvalue(linker_options):
    '''Returns True if sem_getvalue() works on this system, False otherwise.'''
    if IS_MAC:
        # On the Mac, sem_getvalue() exists but always returns -1 (under OS X ≥ 10.9) or
        # ENOSYS ("Function not implemented") under some earlier version(s). It's a waste of
        # time to look for it on that platform.
        return False
    else:
        return does_build_succeed("sniff_sem_getvalue.c", linker_options)


def sniff_sem_timedwait(linker_options):
    '''Returns True if sem_timedwait() works on this system, False otherwise.'''
    return does_build_succeed("sniff_sem_timedwait.c", linker_options)


def sniff_sem_value_max():
    '''Returns either None, or a value suitable for inclusion in system_info.h.'''
    # The max semaphore value should be present in sysconf() on POSIX-compliant systems.
    sem_value_max = maybe_get_sysconf_value('SC_SEM_VALUE_MAX', True)

    if not sem_value_max:
        # This value of last resort should be #defined everywhere. What
        # could possibly go wrong?
        sem_value_max = "_POSIX_SEM_VALUE_MAX"

    return sem_value_max


def sniff_page_size():
    '''Returns the page size (usually 4096 or 16384) suitable for inclusion in system_info.h.

    Raises a DiscoveryError exception if unable to determine the page size.
    '''
    page_size = None

    # When cross compiling under cibuildwheel, I need to rely on their custom env var to set the
    # page size correctly. See https://github.com/osvenskan/posix_ipc/issues/58
    if 'arm' in os.getenv('_PYTHON_HOST_PLATFORM', ''):
        page_size = 16384

    if not page_size:
        # Page size should be present in sysconf() on POSIX-compliant systems, and checking
        # sysconf is easier than invoking the compiler.
        page_size = maybe_get_sysconf_value('SC_PAGESIZE', True)

    if not page_size:
        # OK, I have to do it the hard way. I don't need to worry about linker options here
        # because I'm not calling any functions, just getting the value of a #define.
        page_size = compile_and_run("sniff_page_size.c")

    if not page_size:
        raise DiscoveryError('Unable to determine page size')

    return page_size


def sniff_mq_existence(linker_options):
    '''Returns True if the system supports message queues, False otherwise.'''
    return does_build_succeed("sniff_mq_existence.c", linker_options)


def sniff_mq_prio_max():
    '''Returns the value of MQ_PRIO_MAX, formatted for inclusion in system_info.h.

    Raises a DiscoveryError exception if unable to determine the value.
    '''
    # Max queue priority should be present in sysconf() on POSIX-compliant systems, and checking
    # sysconf is easier than invoking the compiler.
    max_priority = maybe_get_sysconf_value('SC_MQ_PRIO_MAX', True)

    if not max_priority:
        # OK, try to get it via compilation.
        max_priority = compile_and_run("sniff_mq_prio_max.c")

    # Regardless of where I got the value from, it should be int-able (if it's not an int already).
    try:
        max_priority = int(max_priority)
    except Exception:
        # I don't care why the conversion to int failed.
        max_priority = None

    # At this point, max_priority is an int, or None.

    # Under OS X, os.sysconf("SC_MQ_PRIO_MAX") returns -1. This is still true in June 2025 under
    # MacOS 15.5. sniff_mq_prio_max() shouldn't even be called when building on the Mac, but
    # I'll leave this code here because maybe Mac isn't the only platform that behaves that way.
    if max_priority and (max_priority < 0):
        max_priority = None

    if max_priority is None:
        # At this point, I've exhausted all of my options.
        raise DiscoveryError('Unable to determine max message queue priority')

    # Adjust for the fact that these are 0-based values; i.e. permitted
    # priorities range from 0 - (MQ_PRIO_MAX - 1). So why not just make
    # the #define one smaller? Because this one goes up to eleven...
    max_priority -= 1

    # priority is an unsigned int
    return str(max_priority).strip() + "U"


def sniff_mq_max_messages_default():
    '''Returns the value (in bytes) that will be used for the module constant
    QUEUE_MESSAGES_MAX_DEFAULT. This value is only used when creating a MessageQueue; it is the
    default value if the caller doesn't supply one.

    The value returned by this function is formatted for inclusion in system_info.h.

    Since this value is of minor consequence, I return a default value (instead of raising an error)
    if this function can't find a system-supplied value.
    '''
    # This value is not defined by POSIX.

    # On most systems I've tested, msg Qs are implemented via mmap-ed files
    # or a similar interface, so the only theoretical limits are imposed by the
    # file system. In practice, Linux and *BSD impose some fairly tight
    # limits.

    # On Linux it's available in a /proc file and often defaults to the wimpy
    # value of 10.

    # On FreeBSD (and other BSDs, I assume), it's available via sysctl as
    # kern.mqueue.maxmsg. On my FreeBSD 9.1 test system, it defaults to 100.

    # mqueue.h defines mq_attr.mq_maxmsg as a C long, so that's
    # a practical limit for this value.

    # ref: http://linux.die.net/man/7/mq_overview
    # ref: http://www.freebsd.org/cgi/man.cgi?query=mqueuefs&sektion=5&manpath=FreeBSD+7.0-RELEASE
    # http://fxr.watson.org/fxr/source/kern/uipc_mqueue.c?v=FREEBSD91#L195
    # ref: http://groups.google.com/group/comp.unix.solaris/browse_thread/thread/aa223fc7c91f8c38
    # ref: http://cygwin.com/cgi-bin/cvsweb.cgi/src/winsup/cygwin/posix_ipc.cc?cvsroot=src
    # ref: http://cygwin.com/cgi-bin/cvsweb.cgi/src/winsup/cygwin/include/mqueue.h?cvsroot=src
    mq_max_messages = None

    # Try to get the value from where Linux stores it.
    try:
        with open("/proc/sys/fs/mqueue/msg_max") as f:
            mq_max_messages = int(f.read())
    except Exception:
        # Oh well.
        pass

    if not mq_max_messages:
        # Try sysctl
        mq_max_messages = get_sysctl_value('kern.mqueue.maxmsg')
        if mq_max_messages:
            mq_max_messages = int(mq_max_messages)

    if not mq_max_messages:
        # I take a wild guess at an appropriate value. The max possible is > 2 billion, but the
        # values used by Linux and FreeBSD suggest that a smaller default is wiser.
        mq_max_messages = 100

    return mq_max_messages


def sniff_mq_max_message_size_default():
    '''Returns the value (in bytes) that will be used for the module constant
    QUEUE_MESSAGE_SIZE_MAX_DEFAULT. This value is only used when creating a MessageQueue; it is the
    default value if the caller doesn't supply one.

    The value returned by this function is formatted for inclusion in system_info.h.

    Since this value is of minor consequence, I return a default value (instead of raising an error)
    if this function can't find a system-supplied value.
    '''
    # The max message size is not defined by POSIX.

    # On most systems I've tested, msg Qs are implemented via mmap-ed files
    # or a similar interface, so the only theoretical limits are imposed by
    # the file system. In practice, Linux and *BSD impose some tighter limits.

    # On Linux, max message size is available in a /proc file and often
    # defaults to the value of 8192.

    # On FreeBSD (and other BSDs, I assume), it's available via sysctl as
    # kern.mqueue.maxmsgsize. On my FreeBSD 9.1 test system, it defaults to
    # 16384.

    # mqueue.h defines mq_attr.mq_msgsize as a C long, so that's
    # a practical limit for this value.

    # Further complicating things is the fact that the module has to allocate
    # a buffer the size of the queue's max message every time receive() is
    # called, so it would be a bad idea to set this default to the max.
    # I set it to 8192 -- not too small, not too big. I only set it smaller
    # if I'm on a system that tells me I must do so.
    DEFAULT = 8192
    mq_max_message_size_default = 0

    # Try to get the value from where Linux stores it.
    try:
        with open("/proc/sys/fs/mqueue/msgsize_max") as f:
            mq_max_message_size_default = int(f.read())
    except Exception:
        # oh well
        pass

    if not mq_max_message_size_default:
        # Try sysctl
        mq_max_message_size_default = get_sysctl_value('kern.mqueue.maxmsgsize')
        if mq_max_message_size_default:
            mq_max_message_size_default = int(mq_max_message_size_default)

    if not mq_max_message_size_default:
        # Just use the default.
        mq_max_message_size_default = DEFAULT

    return mq_max_message_size_default


def discover():
    '''This is the main entry point for this script. It returns a dict of information it has
    discovered about the buld system. If system_info.h already exists when this script is
    invoked, then the returned dict consists only of one entry which describes whether or not
    posix_ipc needs to be linked to the realtime library.

    If system_info.h does not exist, this script creates and populates it, and the values in the
    returned dict are what it wrote to system_info.h.
    '''
    sys_info = {}

    # First things first -- I need to figure out whether or not the realtime library is needed.
    if IS_MAC:
        # I skip the test under Darwin/Mac/OS X for two reasons. First, I know it isn't needed
        # there. Second, I can't even compile the test for the realtime lib because it
        # references mq_unlink() which OS X doesn't support. Unfortunately sniff_realtime_lib.c
        # *must* reference mq_unlink() or some other mq_xxx() function, because only the message
        # queues need the realtime libs under FreeBSD.
        realtime_lib_is_needed = False
    else:
        # Some platforms (e.g. Linux) require linking to librt
        realtime_lib_is_needed = sniff_realtime_lib()

    sys_info['realtime_lib_is_needed'] = realtime_lib_is_needed

    linker_options = "-lrt" if realtime_lib_is_needed else ""

    if os.path.exists(OUTPUT_FILEPATH):
        # As guaranteed in building.md, this script is (mostly) a no-op if the output file already
        # exists.
        pass
    else:
        # system_info.h doesn't exist, so I need to figure out what values should go into it.

        # Version info is easy. :-)
        with open("VERSION") as f:
            sys_info["POSIX_IPC_VERSION"] = f'"{f.read().strip()}"'

        # Figure out the page size.
        sys_info["PAGE_SIZE"] = sniff_page_size()

        # Sniff sem_getvalue()
        if sniff_sem_getvalue(linker_options):
            sys_info["SEM_GETVALUE_EXISTS"] = ""

        # Sniff sem_timedwait()
        if sniff_sem_timedwait(linker_options):
            sys_info["SEM_TIMEDWAIT_EXISTS"] = ""

        # Sniff the max value of a semaphore.
        sys_info["SEM_VALUE_MAX"] = sniff_sem_value_max()

        # Figure out if message queues are supported at all.
        message_queue_support_exists = sniff_mq_existence(linker_options)

        if message_queue_support_exists:
            sys_info["MESSAGE_QUEUE_SUPPORT_EXISTS"] = ""
            sys_info["QUEUE_MESSAGES_MAX_DEFAULT"] = sniff_mq_max_messages_default()
            sys_info["QUEUE_MESSAGE_SIZE_MAX_DEFAULT"] = sniff_mq_max_message_size_default()
            sys_info["QUEUE_PRIORITY_MAX"] = sniff_mq_prio_max()

        # Turn each of the values in sys_info into lines that will be written to system_info.h
        # PAGE_SIZE and SEM_VALUE_MAX get special handling, and realtime_lib_is_needed doesn't
        # need to go in the header file.
        ignore = ("PAGE_SIZE", "SEM_VALUE_MAX", "realtime_lib_is_needed")
        lines = [f"#define {key} {value}" for key, value in sys_info.items() if key not in ignore]

        # PAGE_SIZE gets some special treatment. It's defined in header files
        # on some systems in which case I might get a redefinition error in
        # my header file, so I wrap it in #ifndef/#endif.
        lines.append("#ifndef PAGE_SIZE")
        lines.append(f"#define PAGE_SIZE {sys_info['PAGE_SIZE']}")
        lines.append("#endif")
        # Ditto for SEM_VALUE_MAX.
        lines.append("#ifndef SEM_VALUE_MAX")
        lines.append(f"#define SEM_VALUE_MAX {sys_info['SEM_VALUE_MAX']}")
        lines.append("#endif")
        # A trailing blank line keeps compilers happy.
        lines.append('')

        msg = """/*
This header file was generated by discover_system_info.py. You can delete it,
edit it, or even write your own. See building.md for details.
*/

"""
        with open(OUTPUT_FILEPATH, 'w') as f:
            f.write(msg + '\n'.join(lines))

    return sys_info


if __name__ == "__main__":
    print(discover())



# Python imports
import unittest
import random
import platform


def _force_int(a_string):
    """Return the string as an int. If it can't be made into an int, return 0."""
    try:
        an_int = int(a_string)
    except (ValueError, TypeError):
        an_int = 0

    return an_int


# Lots of code here to determine if the FreeBSD version is <= 10.2. Those versions contain a
# bug that causes a hang or seg fault if I exercise certain portions of the semaphore tests.
IS_FREEBSD = (platform.system().lower() == 'freebsd')
FREEBSD_VERSION_MINOR = 0
FREEBSD_VERSION_MAJOR = 0

if IS_FREEBSD:
    # I want to get the release number. Here's some samples of what I've seen in platform.release():
    # PC BSD 10.2: '10.2-RELEASE-p14'
    # FreeBSD 9.1: '9.1-RELEASE-p7'
    # I want the number at the beginning. The code below attempts to extract it, but if it runs
    # into anything unexpected it stops trying rather than raising an error.
    release = platform.release().split('-')[0]
    if '.' in release:
        major, minor = release.split('.', 2)
        FREEBSD_VERSION_MAJOR = _force_int(major)
        FREEBSD_VERSION_MINOR = _force_int(minor)
    # else:
        # This isn't in the format I expect, so I don't try to parse it.

# https://bugs.freebsd.org/bugzilla/show_bug.cgi?id=206396
HAS_FREEBSD_BUG_206396 = IS_FREEBSD and (FREEBSD_VERSION_MAJOR <= 10) and \
                         (FREEBSD_VERSION_MINOR <= 2)
FREEBSD_BUG_206396_SKIP_MSG = \
    'Feature buggy on this platform; see https://bugs.freebsd.org/bugzilla/show_bug.cgi?id=206396'


def make_name():
    """Generate a random name suitable for an IPC object."""
    alphabet = 'abcdefghijklmnopqrstuvwxyz'
    return '/' + ''.join(random.sample(alphabet, random.randint(3, 12)))


class Base(unittest.TestCase):
    """Base class for test cases."""
    def assertWriteToReadOnlyPropertyFails(self, target_object, property_name,
                                           value):
        """test that writing to a readonly property raises an exception"""
        # The attributes tested with this code are implemented differently in C.
        # For instance, Semaphore.value is a 'getseters' with a NULL setter,
        # whereas Semaphore.name is a reference into the Semaphore member
        # definition.
        # Under Python 2.6, writing to sem.value raises AttributeError whereas
        # writing to sem.name raises TypeError. Under Python 3, both raise
        # AttributeError (but with different error messages!).
        # This illustrates that Python is a little unpredictable in this
        # matter. Rather than testing each of the numerous combinations of
        # of Python versions and attribute implementation, I just accept
        # both TypeError and AttributeError here.
        # ref: http://bugs.python.org/issue1687163
        # ref: http://bugs.python.org/msg127173
        with self.assertRaises((TypeError, AttributeError)):
            setattr(target_object, property_name, value)



import time
import sys

NULL_CHAR = 0


def say(s):
    """Prints a timestamped, self-identified message"""
    who = sys.argv[0]
    if who.endswith(".py"):
        who = who[:-3]

    s = "%s@%1.6f: %s" % (who, time.time(), s)
    print(s)


def write_to_memory(mapfile, s):
    """Writes the string s to the mapfile"""
    say("writing %s " % s)
    mapfile.seek(0)
    # I append a trailing NULL in case I'm communicating with a C program.
    s += '\0'
    s = s.encode()
    mapfile.write(s)


def read_from_memory(mapfile):
    """Reads a string from the mapfile and returns that string"""
    mapfile.seek(0)
    s = []
    c = mapfile.read_byte()
    while c != NULL_CHAR:
        s.append(c)
        c = mapfile.read_byte()

    s = ''.join([chr(c) for c in s])

    say("read %s" % s)

    return s


def read_params():
    """Reads the contents of params.txt and returns them as a dict"""
    params = {}

    f = open("params.txt")

    for line in f:
        line = line.strip()
        if line:
            if line.startswith('#'):
                pass  # comment in input, ignore
            else:
                name, value = line.split('=')
                name = name.upper().strip()

                if name == "PERMISSIONS":
                    # Think octal, young man!
                    value = int(value, 8)
                elif "NAME" in name:
                    # This is a string; leave it alone.
                    pass
                else:
                    value = int(value)

                # print "name = %s, value = %d" % (name, value)

                params[name] = value

    f.close()

    return params



import setuptools
from setuptools.extension import Extension
import sys

# When python -m build runs, sys.path contains a minimum of entries. I add the current directory
# to it (which is guaranteed by setuptools to be the project's root) so that I can import my
# build_support tools.
sys.path.append('.')
import build_support.discover_system_info

# As of April 2025, specifying the license metadata here (rather than in pyproject.toml) seems
# like the best solution for now. See https://github.com/osvenskan/posix_ipc/issues/68
LICENSE = "BSD-3-Clause"

# As of April 2025, use of tool.setuptools.ext-modules is stil experimental in pyproject.toml.
# Also, this code needs to dynamically adjust the `libraries` value that's passed to setuptools,
# so I can't get rid of setup.py just yet.
SOURCE_FILES = ["src/posix_ipc_module.c"]
DEPENDS = ["src/posix_ipc_module.c", "src/system_info.h"]

libraries = []

system_info = build_support.discover_system_info.discover()

# Linux requires linking against the realtime libs. My notes say that FreeBSD also required this,
# but it looks like librt on FreeBSD is a raytracing library so maybe not. :-) In any case,
# adding "rt" to the list of linked libraries on platforms where it doesn't exist (e.g. Mac)
# causes a build failure (due to a link error), so I need to be careful about only adding it
# where it's needed.
if system_info['realtime_lib_is_needed']:
    libraries.append("rt")

ext_modules = [Extension("posix_ipc",
                         SOURCE_FILES,
                         libraries=libraries,
                         depends=DEPENDS,
                         # -E is useful for debugging compile errors.
                         # extra_compile_args=['-E'],
                         )]

setuptools.setup(ext_modules=ext_modules,
                 license=LICENSE,
                 )



import time
import sys


def say(s):
    who = sys.argv[0]
    if who.endswith(".py"):
        who = who[:-3]

    s = "%s@%1.6f: %s" % (who, time.time(), s)
    print(s)


def read_params():
    params = {}

    f = open("params.txt")

    for line in f:
        line = line.strip()
        if len(line):
            if line.startswith('#'):
                pass  # comment in input, ignore
            else:
                name, value = line.split('=')
                name = name.upper().strip()

                if name == "PERMISSIONS":
                    value = int(value, 8)
                elif "NAME" in name:
                    # This is a string; leave it alone.
                    pass
                else:
                    value = int(value)

                # print("name = %s, value = %d" % (name, value))

                params[name] = value

    f.close()

    return params
